@@ -6,7 +6,7 @@
  */
 /*
  *  This file is part of adns, which is
- *    Copyright (C) 1997-2000,2003,2006,2014-2016  Ian Jackson
+ *    Copyright (C) 1997-2000,2003,2006,2014-2016,2020  Ian Jackson
  *    Copyright (C) 2014  Mark Wooding
  *    Copyright (C) 1999-2000,2003,2006  Tony Finch
  *    Copyright (C) 1991 Massachusetts Institute of Technology
@@ -53,7 +53,7 @@ typedef unsigned char byte;
 
 /* Configuration and constants */
 
-#define MAXSERVERS 5
+#define MAXSERVERS 5 /* do not increase beyond no. of bits in `unsigned'! */
 #define MAXSORTLIST 15
 #define UDPMAXRETRIES 15
 #define UDPRETRYMS 2000
@@ -95,7 +95,8 @@ typedef unsigned char byte;
 
 typedef enum {
   cc_user,
-  cc_entex,
+  cc_enter,
+  cc_exit,
   cc_freq
 } consistency_checks;
 
@@ -170,7 +171,7 @@ typedef struct typeinfo {
    * Previously, used alloc_interim, now use alloc_final.
    */
 
-  adns_status (*convstring)(vbuf *vb, const void *data);
+  adns_status (*convstring)(vbuf *vb, adns_rrtype, const void *data);
   /* Converts the RR data to a string representation in vbuf.
    * vbuf will be appended to (it must have been initialised),
    * and will not be null-terminated by convstring.
@@ -246,7 +247,7 @@ union maxalign {
   void *p;
   void (*fp)(void);
   union maxalign *up;
-} data;
+};
 
 struct adns__query {
   adns_state ads;
@@ -371,7 +372,8 @@ struct adns__state {
   int configerrno;
   struct query_queue udpw, tcpw, childw, output, intdone;
   adns_query forallnext;
-  int nextid, tcpsocket;
+  unsigned nextid;
+  int tcpsocket;
   struct udpsocket { int af; int fd; } udpsockets[MAXUDP];
   int nudpsockets;
   vbuf tcpsend, tcprecv;
@@ -719,6 +721,8 @@ void adns__returning(adns_state ads, adns_query qu);
  * external-faciing functions which call adns__returning should
  * normally be avoided in internal code. */
 
+void adns__intdone_process(adns_state ads);
+
 /* From reply.c: */
 
 void adns__procdgram(adns_state ads, const byte *dgram, int len,
@@ -934,7 +938,7 @@ static inline int errno_resources(int e) { return e==ENOMEM || e==ENOBUFS; }
   (( ((sz)+sizeof(union maxalign)-1) / sizeof(union maxalign) )	\
    * sizeof(union maxalign) )
 
-#define GETIL_B(cb) (((dgram)[(cb)++]) & 0x0ff)
+#define GETIL_B(cb) (((dgram)[(cb)++]) & 0x0ffu)
 #define GET_B(cb,tv) ((tv)= GETIL_B((cb)))
 #define GET_W(cb,tv) ((tv)=0,(tv)|=(GETIL_B((cb))<<8), (tv)|=GETIL_B(cb), (tv))
 #define GET_L(cb,tv) ( (tv)=0,				\
